#[derive(Debug, Default, Clone, Copy, PartialEq)]
pub struct Pos {
	pub x: isize,
	pub y: isize,
}

impl From<(usize, usize)> for Pos {
	fn from(value: (usize, usize)) -> Self {
		Self {
			x: value.0 as isize,
			y: value.1 as isize,
		}
	}
}

#[derive(Debug)]
pub struct Grid<T> {
	rows: Vec<Vec<T>>,
	width: usize,
	height: usize,
}

impl<T> Grid<T>
where
	T: Default + Copy,
{
	pub fn new(rows: Vec<Vec<T>>) -> Self {
		Self {
			width: rows[0].len(),
			height: rows.len(),
			rows,
		}
	}

	pub fn in_bounds(&self, p: Pos) -> bool {
		p.x >= 0 && p.y >= 0 && p.x < self.width as isize && p.y < self.height as isize
	}

	pub fn get(&self, p: Pos) -> T {
		if self.in_bounds(p) {
			self.rows[p.y as usize][p.x as usize]
		} else {
			T::default()
		}
	}

	pub fn get_mut(&mut self, p: Pos) -> &mut T {
		if self.in_bounds(p) {
			&mut self.rows[p.y as usize][p.x as usize]
		} else {
			panic!(
				"position {p:?} out of bounds, size is {}x{}",
				self.width, self.height
			);
		}
	}

	pub fn width(&self) -> usize {
		self.width
	}

	pub fn height(&self) -> usize {
		self.height
	}
}
